/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplyDatasourcePropsConfirmAction.java,v 1.5 2003/08/22 15:36:27 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.resource;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;
import org.objectweb.jonas.webapp.jonasadmin.deploy.BaseDeployAction;

/**
 * @author Michel-Ange ANTON
 */

public class ApplyDatasourcePropsConfirmAction extends BaseDeployAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Form used
        DatasourcePropertiesForm oForm = (DatasourcePropertiesForm) m_Session.getAttribute(
            "datasourcePropertiesForm");
        try {
            // populate and deploy (load resource)
            populate(oForm);
            // refresh tree
            m_WhereAreYou.setCurrentJonasDeploymentType(WhereAreYou.DEPLOYMENT_DATASOURCE);
            refreshTree(p_Request);
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Remove form
        m_Session.removeAttribute("datasourcePropertiesForm");
        // Forward to the jsp.
        return (p_Mapping.findForward("ActionListDatasources"));
    }

// --------------------------------------------------------- Protected Methods

    private static final String DEF_CLASSNAME = "no class name";
    private static final String DEF_URL = "no url";
    private static final String DEF_DESCRIPTION = "no desc";
    private static final String DEF_USERNAME = "";
    private static final String DEF_PASSWORD = "";
    private static final String DEF_FACTORY = "none";
    //private static final String DEF_XADATANAME = "none";
    //private static final String DEF_ISOLATIONLEVEL = "";
    private static final String DEF_MAPPERNAME = "rdb";

    protected void populate(DatasourcePropertiesForm p_Form)
        throws Exception {
        /*
               #set ( $val = $datasourceProperties.getProperty("datasource.name", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.description", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.url", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.classname", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.username", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.password", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.factory", "none") )
               #set ( $val = $datasourceProperties.getProperty("datasource.mapper", "") )
               #set ( $val = $datasourceProperties.getProperty("datasource.connmaxage", "30") )
               #set ( $val = $datasourceProperties.getProperty("datasource.connchecklevel", "0") )
             #set ( $val = $datasourceProperties.getProperty("datasource.connteststmt", "SELECT 1") )
               #set ( $val = $datasourceProperties.getProperty("datasource.minconpool", "0") )
             #set ( $val = $datasourceProperties.getProperty("datasource.maxconpool", "no max boundary") )
         */

        Properties oProps = new Properties();
        setStringAttribute(oProps, "datasource.name", p_Form.getDatasourceName());
        setStringAttribute(oProps, "datasource.description", p_Form.getDatasourceDescription()
            , DEF_DESCRIPTION);
        setStringAttribute(oProps, "datasource.url", p_Form.getDatasourceUrl(), DEF_URL);
        setStringAttribute(oProps, "datasource.classname", p_Form.getDatasourceClassname()
            , DEF_CLASSNAME);
        setStringAttribute(oProps, "datasource.username", p_Form.getDatasourceUsername()
            , DEF_USERNAME);
        setStringAttribute(oProps, "datasource.password", p_Form.getDatasourcePassword()
            , DEF_PASSWORD);
        setStringAttribute(oProps, "datasource.factory", p_Form.getDatasourceFactory(), DEF_FACTORY);
//      setStringAttribute(oProps, "datasource.xadataname", p_Form.getDatasourceXadataname()
//         , DEF_XADATANAME);
//      setStringAttribute(oProps, "datasource.mincon", p_Form.getDatasourceMincon());
//      setStringAttribute(oProps, "datasource.maxcon", p_Form.getDatasourceMaxcon());
//      setStringAttribute(oProps, "datasource.isolationlevel", p_Form.getDatasourceIsolationlevel()
//         , DEF_ISOLATIONLEVEL);
        setStringAttribute(oProps, "datasource.mapper", p_Form.getDatasourceMapper()
            , DEF_MAPPERNAME);
        setStringAttribute(oProps, "jdbc.connmaxage", p_Form.getJdbcConnmaxage());
        setStringAttribute(oProps, "jdbc.connchecklevel", p_Form.getJdbcConnchecklevel());
        setStringAttribute(oProps, "jdbc.connteststmt", p_Form.getJdbcConnteststmt());
//      setStringAttribute(oProps, "jdbc.connexcltimeout", p_Form.getJdbcConnexcltimeout());
//      setStringAttribute(oProps, "jdbc.connmaxthreads", p_Form.getJdbcConnmaxthreads());
        setStringAttribute(oProps, "jdbc.minconpool", p_Form.getJdbcMinconpool());
        setStringAttribute(oProps, "jdbc.maxconpool", p_Form.getJdbcMaxconpool());
//      setStringAttribute(oProps, "jdbc.sleeptimepool", p_Form.getJdbcSleeptimepool());
//      setBooleanAttribute(oProps, "jdbcGcpool", p_Form.getJdbcGcpool());
//      setStringAttribute(oProps, "jdbc.gGenerationpool", p_Form.getJdbcGenerationpool());
//      setStringAttribute(oProps, "jdbc.deadlockpool", p_Form.getJdbcDeadlockpool());
//      setStringAttribute(oProps, "jdbc.loopwaitpool", p_Form.getJdbcLoopwaitpool());

        // Load Datasource
        Object[] aoParam = {
            p_Form.getName(), oProps, new Boolean(false)};
        String[] asSign_3 = {
            "java.lang.String", "java.util.Properties", "java.lang.Boolean"};
        JonasManagementRepr.invoke(JonasObjectName.databaseService(), "loadDataSource", aoParam
            , asSign_3);

    }

}
