/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ConfigGenerator.java,v 1.5 2003/09/17 09:30:05 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen;

import org.objectweb.jonas_ws.deployment.api.ServiceDesc;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.ServiceRef;

import java.util.List;
import java.util.Iterator;


/**
 * This interface provides methods used to generate configuration files for a
 * Web services Engine.
 */
public interface ConfigGenerator {

    // configutator option names.
    public final static  String VERBOSE = "verbose";
    public final static  String DEBUG = "debug";
    public final static  String NO_CONFIG = "noConfig";
    public final static  String KEEP_GEN = "keepGen";

    // options default values.
    public final static  Boolean VERBOSE_DEFAULT = new Boolean(false);
    public final static  Boolean DEBUG_DEFAULT = new Boolean(false);
    public final static  Boolean NO_CONFIG_DEFAULT = new Boolean(false);

    /**
     * Return the webservices engine class name.
     * @return the webservices engine class name.
     *
     * @throws WsGenException if the method isn't implemented
     */
    public String getEngineWebApp()
	throws WsGenException;

    public String getDefaultMapping();

    /**
     * Generate endpoint configuration files for the set of ServiceDesc
     * included in WSDeploymentDesc. Common method for all Web Services
     * implementation (Glue/Axis).
     *
     * @param dd The DeploymentDesc to process (image of a webservices.xml)
     * @param filename The war or jar being processed
     *
     * @throws WsGenException if generation fails
     */
    void genEndpointConfig(WSDeploymentDesc dd,
			   String filename)
	throws WsGenException;

    /**
     * Generate client configuration files for the list of ServiceRef.
     * Common method for all Web Services implementation (Glue/Axis).
     *
     * @param servRefs The list of ServiceRef to process.
     * @param filename The war or jar being processed
     *
     * @throws WsGenException if generation fails
     */
    void genClientConfig(ServiceRef[] servRefs,
			 String filename)
	throws WsGenException;

    /**
     * Generate client configuration files for each ServiceRef of bean descs.
     * Common method for all Web Services implementation (Glue/Axis).
     *
     * @param bdIt an iterator on a bean desc list
     * @param filename The war or jar being processed
     *
     * @throws WsGenException if generation fails
     */
    void genClientConfig(Iterator bdIt,
			 String filename)
	throws WsGenException;

    /**
     * Set an option for generation.
     * @param name option name
     * @param value option value
     */
    public void setOption(String name, Object value);


    /**
     * Get an option, return the option value or default
     * if the option isn't set.
     * @param name option name
     * @param defolt option value
     *
     * @return the option value or default
     * if the option isn't set.
     */
    public Object getOption(String name, Object defolt);

    /**
     * Get an option, return the option value or null
     * if the option isn't set.
     * @param name option name
     *
     * @return the option value or null
     * if the option isn't set.
     */
    public Object getOption(String name);

}
