/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WarManager.java,v 1.6 2003/09/19 15:07:47 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen;

//import java
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarException;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

//import xml
import org.objectweb.jonas_ws.wsgen.xml.WebXml;

//import jonas_lib
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Use xml classes to get and write
 * informations inside a war file.
 */
public class WarManager extends JarManager {

    private static final String DEFAULT_WEB = "web.xml";
    private static final String DEFAULT_JONAS_WEB = "jonas-web.xml";

    private static final String WEB = "WEB-INF" + File.separator + DEFAULT_WEB;
    private static final String JONAS_WEB = "WEB-INF" + File.separator + DEFAULT_JONAS_WEB;


    /** jonas-web.xml */
//    private JonasWebXml jweb = null;
    /** web.xml */
    private WebXml      web  = null;

    /**
     * Build a new instance of a WarManager.
     *
     * @param warFilename the name of the war file
     * that we want to manage.
     *
     * @throws WsGenException if the war file doesn't exist.
     */
    public WarManager (String warFilename) 
	throws WsGenException {
	super(warFilename);
	// get the war file DD.
	if (!isNewFile()){
	    try {
		JarFile jarFile = new JarFile(this.jarFileName);
		InputStream is = jarFile.getInputStream(jarFile.getEntry(WEB));
		web = loadWebApp(is, WEB);
		is.close();		
	    }
	    catch (IOException ioe) {
		if (logger.isLoggable(BasicLevel.WARN))
		    logger.log(BasicLevel.WARN, "Cannot get '" + WEB +
			       "' JarEntry from '" + jarFileName + "'\n" +
			       "will continue with default web.xml");	
		web = createDefaultWebDesc();
	    }  
	}else{
	    System.out.println("It's a new file : create a default web desc");
	    web = createDefaultWebDesc();
	}
    }


    /**
     * Add a new Servlet in the current war file.
     * Temporary (no tests name redundance : servlet-name).
     * @todo add ejb-ref / ejb-local-ref for minimum security
     *
     * @param className the new servlet class name.
     * @param mappings List of servlet-mapping
     *
     * @throws WsGenException
     */
    public void addServlet (String className, List mappings)
	throws WsGenException {

	// get the servlet name (only last part)
	String sName = className.substring(className.lastIndexOf(".")+1, className.length());
	web.addServlet(sName, className);

	// Create the MappingElement(s)
	for (Iterator i = mappings.iterator() ; i.hasNext();){	    
	    String map = (String) i.next();
	    web.addServletMapping(sName, map);
	}
	
    }

    protected void save() throws WsGenException {
	
	if (!isNewFile()){
	    // remove the web dd entrie in temp jar entries
	    Vector validEntries = new Vector();		
	    for (Enumeration e = tjEntries; e.hasMoreElements();) {		    
		ZipEntry ze = (ZipEntry)(e.nextElement());
		if(!ze.getName().equals(WEB)){		    
		    validEntries.add(ze);
		}
	    }
	    tjEntries = validEntries.elements();
	}
	
	super.save();
	try {
	    jos.putNextEntry(new JarEntry(WEB));
	    web.marshal(jos);
	}
	catch (NullPointerException npe) {
	    throw new WsGenException("The given file name '" + WEB + "' is null",npe);
	}
	catch (IllegalArgumentException iae) {
	    throw new WsGenException("The given file name '" + WEB + "' is too long",iae);
	}
	catch (JarException jare) {
	    throw new WsGenException("Jar entry format error for '" + WEB + "'", jare);
	}
	catch (IOException ioe) {
	    throw new WsGenException("Error when adding jar entry '" + WEB + "'", ioe);
	}
    }
    
    private WebXml createDefaultWebDesc() throws WsGenException{

	if (logger.isLoggable(BasicLevel.DEBUG))
	    logger.log(BasicLevel.DEBUG, "Create Default web.xml for '" + jarFileName + "'");

        String jonasRoot = System.getProperty("install.root");
        if (jonasRoot == null) {
            throw new WsGenException("System property 'install.root' not set");
        }
        String path2Tmpl = new String(jonasRoot + File.separatorChar + "templates"
                                      + File.separatorChar + "wsgen");
	InputStream is = null;
	String filename = path2Tmpl + File.separatorChar + DEFAULT_WEB;
	WebXml webapp;
	try {
	    is = new FileInputStream(filename);
	    webapp = new WebXml (is);
	    is.close();
	}
	catch (Exception e) {
            throw new WsGenException("Error loading web dd '"+filename+"'",e);	    
	}

	return webapp;
    }
/*
    private JonasWebXml createDefaultJonasWebDesc() throws WsGenException{

	if (logger.isLoggable(BasicLevel.DEBUG))
	    logger.log(BasicLevel.DEBUG, "Create Default jonas-web.xml for '"
		       + jarFileName + "'");
	
        String jonasRoot = System.getProperty("install.root");
        if (jonasRoot == null) {
            throw new WsGenException("System property 'install.root' not set");
        }	
        String path2Tmpl = new String(jonasRoot + File.separatorChar + "templates"
                                      + File.separatorChar + "wsgen");
	InputStream is = null;
	String filename = path2Tmpl + File.separatorChar + DEFAULT_JONAS_WEB;
	try {
	    is = new FileInputStream(filename);
	    // JonasWebXml webapp = new WebXml (is);
	    is.close();
	}
	catch (Exception e) {
            throw new WsGenException("Error loading jonas-web dd '"+filename+"'",e);	    
	}
	// todo : get a JonasWebXml object
	return null;
    }
*/
    /** 
     * Load the web.xml file.
     * @param is the InputStream of the XML file.
     * @param fileName the name of the file (web.xml).
     * @return a structure containing the result of the web.xml parsing.
     * @throws WebContainerDeploymentDescException if the deployment descriptor 
     * is corrupted.
     */
    private static WebXml loadWebApp(InputStream is, String fileName) 
        throws WsGenException {
        return  new WebXml(is);
    }

    /**
     * Load the jonas-web.xml file.
     * @param is the InputStream of the XML file.
     * @param fileName the name of the file (jonas-web.xml).
     * @return a structure containing the result of the jonas-web.xml parsing.
     * @throws WsGenException if the deployment 
     * descriptor is corrupted.
     */
/*
    private static JonasWebXml loadJonasWebApp(InputStream is, 
                                               String fileName) 
        throws WsGenException { 
	// todo : get the JonasWebXml object
	return null; 
    }
*/
}
