/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WebXml.java,v 1.2 2003/09/19 15:07:48 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen.xml;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.LineSeparator;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

import java.io.FileWriter;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import org.objectweb.jonas_lib.deployment.SAXParserHelper;

public class WebXml extends Xml {

    private static String SERVICE_REF_TAG = "service-ref";
    private static String SERVICE_INTERFACE_TAG = "service-interface";
    private static String SERVICE_REF_NAME_TAG = "service-ref-name";

    private static String SERVLET_TAG = "servlet";
    private static String SERVLET_NAME_TAG = "servlet-name";
    private static String SERVLET_CLASS_TAG = "servlet-class";

    private static String MAP_TAG = "servlet-mapping";
    private static String MAP_URL_TAG = "url-pattern";
    private static String MAP_NAME_TAG = "servlet-name";
    
    public WebXml (InputStream is) throws XmlException {
	super(is);
    }

    public void addServletMapping(String servName, String url){


	Document core = getDocument();
	Element root = core.getDocumentElement();
	NodeList nl = root.getElementsByTagName(MAP_TAG);

	// build nodes
	Node mapNode = nl.item(0);
	Node servNameTxt = core.createTextNode(servName);
	Node urlTxt = core.createTextNode(url);
	Element servNameE = core.createElement(MAP_NAME_TAG);
	Element urlE = core.createElement(MAP_URL_TAG);
	Element newNode = core.createElement(MAP_TAG);

	servNameE.appendChild(servNameTxt);
	urlE.appendChild(urlTxt);
	newNode.appendChild(servNameE);
	newNode.appendChild(urlE);
	root.insertBefore(newNode, mapNode);	
    }

    public void addServlet(String servName, String servClass){

	Document core = getDocument();
	Element root = core.getDocumentElement();
	NodeList nl = root.getElementsByTagName(SERVLET_TAG);

	// build nodes
	Node servletNode = nl.item(0);
	Node servNameTxt = core.createTextNode(servName);
	Node servClassTxt = core.createTextNode(servClass);
	Element servNameE = core.createElement(SERVLET_NAME_TAG);
	Element servClassE = core.createElement(SERVLET_CLASS_TAG);
	Element newNode = core.createElement(SERVLET_TAG);

	servNameE.appendChild(servNameTxt);
	servClassE.appendChild(servClassTxt);
	newNode.appendChild(servNameE);
	newNode.appendChild(servClassE);
	root.insertBefore(newNode, servletNode);

    }

    /**
     * Change or create ?
     */
    public void changeServiceEndpointInterface(String serviceRefName,
					       String newInterfaceName) {
	Document core = getDocument();
	Element root = core.getDocumentElement();
	NodeList nl = root.getElementsByTagName(SERVICE_REF_TAG);

	// Search the serviceRefName in all service-ref
	Element sRef = null;
	/* service-ref
	 *  service-ref-name
	 *   #text
	 */
	for (int i = 0; i < nl.getLength() && sRef == null; i++) {
	    Element srn = ((Element) (((Element) nl.item(i)).getElementsByTagName(SERVICE_REF_NAME_TAG)).item(0));
	    if (srn.getNodeValue().equals(serviceRefName)) {
		sRef = (Element) nl.item(i);
	    }
	}

	if (sRef != null) {

	    NodeList intf = sRef.getElementsByTagName(SERVICE_INTERFACE_TAG);
	    Element serviceInterface = (Element) intf.item(0);
	    serviceInterface.getFirstChild().setNodeValue(newInterfaceName);

	} else {
	    // log ?
	}

    }
    
}
