/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: Xml.java,v 1.2 2003/09/19 15:07:48 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen.xml;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.LineSeparator;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;

import java.io.FileWriter;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import org.objectweb.jonas_lib.deployment.SAXParserHelper;

public class Xml {

    private Document core;
    
    public Xml (InputStream is) throws XmlException {
	read(is);
    }
    
    /**
     * Parse the given xml input stream and build the dom tree.
     * @param is the input stream on a xml file.
     */
    protected void read (InputStream is)
	throws XmlException {
	try {
	    // load and parse the document
	    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	    dbf.setValidating(true);
	    DocumentBuilder docBuilder = dbf.newDocumentBuilder();
	    docBuilder.setEntityResolver(new SAXParserHelper());
	    core = docBuilder.parse(is);
	}
	catch (Exception e) {
	    throw new XmlException("Error when reading the given input stream",e);
	}
    }
    /**
     * Write the modified document in the given output stream.
     * @param os the output stream to write the document.
     */
    public void marshal(OutputStream os)
	throws XmlException {
	try {
	    // define the format for the xml document
	    OutputFormat format = new OutputFormat(core);
	    format.setLineSeparator(LineSeparator.Unix);
	    format.setIndenting(true);
	    format.setLineWidth(0);             
	    format.setPreserveSpace(true);
	    
	    // document serialization and writing
	    XMLSerializer serializer = new XMLSerializer (os, format);
	    serializer.asDOMSerializer();
	    serializer.serialize(core);
	}
	catch (Exception e) {
	    throw new XmlException("Error when writing in the given output stream",e);	    
	}
    }
    
    protected Document getDocument() {
	return core;
    }
}
